package edu.rice.comp322;

import edu.rice.hj.api.SuspendableException;

import java.io.IOException;
import java.util.List;
import java.util.PriorityQueue;
import java.util.Queue;

/**
 * @author <a href="http://shams.web.rice.edu/">Shams Imam</a> (shams@rice.edu)
 */
public class SequentialBenchmark extends Benchmark {

    public static void main(final String[] args) {
        BenchmarkRunner.runBenchmark(args, new SequentialBenchmark());
    }

    protected ProblemState solution;

    protected SequentialBenchmark() {
        super();
    }

    @Override
    public void initialize(final String[] args) throws IOException {
        SudokuConfig.parseArgs(args);
        solution = null;
    }

    @Override
    public void printArgInfo() {
        SudokuConfig.printArgs();
    }

    @Override
    public void runIteration() throws SuspendableException {
        computeSudoku();
    }

    @Override
    public void cleanupIteration(final boolean lastIteration, final double execTimeMillis) {

        SudokuConfig.validate(this, solution);

        if (!lastIteration) {
            solution = null;
        } else {
            System.out.println(solution);
        }
        System.out.println();
    }

    @Override
    protected ProblemState computeSudoku() throws SuspendableException {

        final Queue<ProblemState> queue = new PriorityQueue<>(SudokuUtils.boardComparator());

        // Create root node
        queue.add(SudokuConfig.initialProblemState);

        computeSudoku(queue);

        if (solution != null) {
            System.out.printf(BenchmarkRunner.argOutputFormat, "Num Moves to Solution", solution.numMoves);
        }
        return solution;
    }

    private void computeSudoku(final Queue<ProblemState> queue) throws SuspendableException {

        while (!queue.isEmpty()) {

            final ProblemState loopNode = queue.poll();

            processNode(queue, loopNode);
            if (solution != null) {
                return;
            }
        }
    }

    private void processNode(final Queue<ProblemState> queue, final ProblemState loopNode) {
        if (loopNode.isSolution()) {
            solution = loopNode;
            return;
        }

        final List<ProblemState> neighbors = loopNode.neighbors();
        for (final ProblemState neighbor : neighbors) {
            if (neighbor.isConsistent()) {
                queue.add(neighbor);
            }
        }
    }

}
