package edu.rice.comp322;

import junit.framework.TestCase;

import java.util.*;

/**
 * This is a test class for your homework and should not be modified.
 *
 * @author <a href="http://shams.web.rice.edu/">Shams Imam</a> (shams@rice.edu)
 * @author Vivek Sarkar (vsarkar@rice.edu)
 */
public class Homework4SudokuUtilsTest extends TestCase {

    public void testSolutionBoard() {
        System.out.println("Homework4SudokuUtilsTest.testSolutionBoard: starts...");

        final int[] boardValues = new int[]{
                1, 2, 3, 0,
                3, 0, 2, 1,
                2, 1, 0, 3,
                0, 3, 1, 2,
        };
        final ProblemState problemState = new ProblemState(boardValues);

        assertTrue("Board expected to be consistent, but implementation says inconsistent!", problemState.isConsistent());
        assertTrue("Board expected to be a solution, but implementation says not a solution!", problemState.isSolution());

        System.out.println("Homework4SudokuUtilsTest.testSolutionBoard: ends.");
    }

    public void testConsistentBoard() {
        System.out.println("Homework4SudokuUtilsTest.testConsistentBoard: starts...");

        final int[] boardValues = new int[]{
                1, -1, -1, -1,
                3, 0, -1, -1,
                2, -1, 0, 3,
                0, 3, -1, 1,
        };
        final ProblemState problemState = new ProblemState(boardValues);

        assertTrue("Board expected to be consistent, but implementation says inconsistent!", problemState.isConsistent());
        assertTrue("Board expected not be a solution, but implementation says a solution!", !problemState.isSolution());

        System.out.println("Homework4SudokuUtilsTest.testConsistentBoard: ends.");
    }

    public void testSetPossibleValue1() {
        System.out.println("Homework4SudokuUtilsTest.testSetPossibleValue1: starts...");

        final int[] boardValues = new int[]{
                1, -1, -1, -1,
                3, 0, -1, -1,
                2, -1, 0, 3,
                0, 3, -1, 1,
        };
        final ProblemState problemState = new ProblemState(boardValues);
        final ProblemState newBoard = problemState.setPossibleValue(1, 2);

        final String actualString = newBoard.boardString().replaceAll("\\s+", "").trim();
        final String expectedString = " 1 2 3 0\n 3 0 . 2\n 2 1 0 3\n 0 3 2 1".replaceAll("\\s+", "").trim();

        assertTrue("setPossibleValue() returned incorrect result!", expectedString.equals(actualString));

        System.out.println("Homework4SudokuUtilsTest.testSetPossibleValue1: ends.");
    }

    public void testSetPossibleValue2() {
        System.out.println("Homework4SudokuUtilsTest.testSetPossibleValue2: starts...");

        final int[] boardValues = new int[]{
                1, -1, -1, -1,
                3, 0, -1, -1,
                2, -1, 0, 3,
                0, 3, -1, 1,
        };
        final ProblemState problemState = new ProblemState(boardValues);
        final ProblemState newBoard = problemState.setPossibleValue(2, 3);

        final String actualString = newBoard.boardString().replaceAll("\\s+", "").trim();
        final String expectedString = " 1 2 3 0\n 3 0 . 2\n 2 1 0 3\n 0 3 2 1".replaceAll("\\s+", "").trim();

        assertTrue("setPossibleValue() returned incorrect result!", expectedString.equals(actualString));

        System.out.println("Homework4SudokuUtilsTest.testSetPossibleValue2: ends.");
    }

    public void testCountUnsolved1() {
        System.out.println("Homework4SudokuUtilsTest.testCountUnsolved1: starts...");

        final int[] boardValues = new int[]{
                1, 2, 3, 0,
                3, 0, 2, 1,
                2, 1, 0, 3,
                0, 3, 2, 1,
        };
        final ProblemState problemState = new ProblemState(boardValues);
        final int actualUnsolved = problemState.numUnsolved;
        final int expectedUnsolved = 0;

        assertEquals("Expected " + expectedUnsolved + ", actual " + actualUnsolved, expectedUnsolved, actualUnsolved);

        System.out.println("Homework4SudokuUtilsTest.testCountUnsolved2: ends.");
    }

    public void testCountUnsolved2() {
        System.out.println("Homework4SudokuUtilsTest.testCountUnsolved2: starts...");

        final int[] boardValues = new int[]{
                1, -1, -1, -1,
                3, 0, -1, -1,
                2, -1, 0, 3,
                0, 3, -1, 1,
        };
        final ProblemState problemState = new ProblemState(boardValues);
        final int actualUnsolved = problemState.numUnsolved;
        final int expectedUnsolved = 1;

        assertEquals("Expected " + expectedUnsolved + ", actual " + actualUnsolved, expectedUnsolved, actualUnsolved);

        System.out.println("Homework4SudokuUtilsTest.testCountUnsolved2: ends.");
    }

    public void testWorkQueue() {
        System.out.println("Homework4SudokuUtilsTest.testWorkQueue: starts...");

        final Comparator<Integer> c = Integer::compareTo;
        final Queue<Integer> workQueue = SudokuUtils.newWorkQueue(c);

        workQueue.addAll(Arrays.asList(3, 1, 6, 2, 7, 4, 5));

        final List<Integer> actualList = new ArrayList<>();
        while (!workQueue.isEmpty()) {
            actualList.add(workQueue.poll());
        }

        final List<Integer> expectedList = Arrays.asList(1,2,3,4,5,6,7);

        assertTrue("Expected " + expectedList + ", actual " + actualList, expectedList.equals(actualList));

        System.out.println("Homework4SudokuUtilsTest.testWorkQueue: ends.");
    }

    public void testSplitTaskQueue() {
        System.out.println("Homework4SudokuUtilsTest.testSplitTaskQueue: starts...");

        final Comparator<Integer> c = Integer::compareTo;
        final Queue<Integer> workQueue = new LinkedList<>(Arrays.asList(3, 1, 6, 2, 7, 4, 5));

        final Queue<Integer>[] queues = SudokuUtils.splitTaskQueue(workQueue, c, 2);
        assertTrue("Result is null!", queues != null);
        assertEquals("Did not create 2 queues!", 2, queues.length);

        final List<Integer> actualList = new ArrayList<>();
        actualList.addAll(queues[0]);
        actualList.addAll(queues[1]);
        Collections.sort(actualList);

        assertTrue("queues[0] is empty!" + actualList, !queues[0].isEmpty());
        assertTrue("queues[1] is empty!" + actualList, !queues[1].isEmpty());

        final List<Integer> expectedList = new ArrayList<>();
        expectedList.addAll(Arrays.asList(1, 2, 3, 4, 5, 6, 7));

        assertTrue("Expected " + expectedList + ", actual " + actualList, expectedList.equals(actualList));

        System.out.println("Homework4SudokuUtilsTest.testSplitTaskQueue: ends.");
    }


}
